#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of convertdate.
# http://github.com/fitnr/convertdate

# Licensed under the GPL-v3.0 license:
# http://opensource.org/licenses/MIT
# Copyright (c) 2016, fitnr <fitnr@fakeisthenewreal>

# source: Wikipedia
# month 13 is the Sansculottides
french_republican_days = {
    1: [
        u"Raisin",
        u"Safran",
        u"Châtaigne",
        u"Colchique",
        u"Cheval",
        u"Balsamine",
        u"Carotte",
        u"Amaranthe",
        u"Panais",
        u"Cuve",
        u"Pomme de terre",
        u"Immortelle",
        u"Potiron",
        u"Réséda",
        u"Âne",
        u"Belle de nuit",
        u"Citrouille",
        u"Sarrasin",
        u"Tournesol",
        u"Pressoir",
        u"Chanvre",
        u"Pêche",
        u"Navet",
        u"Amaryllis",
        u"Bœuf",
        u"Aubergine",
        u"Piment",
        u"Tomate",
        u"Orge",
        u"Tonneau",
    ],
    2: [
        u"Pomme",
        u"Céleri",
        u"Poire",
        u"Betterave",
        u"Oie",
        u"Héliotrope",
        u"Figue",
        u"Scorsonère",
        u"Alisier",
        u"Charrue",
        u"Salsifis",
        u"Mâcre",
        u"Topinambour",
        u"Endive",
        u"Dindon",
        u"Chervis",
        u"Cresson",
        u"Dentelaire",
        u"Grenade",
        u"Herse",
        u"Bacchante",
        u"Azerole",
        u"Garance",
        u"Orange",
        u"Faisan",
        u"Pistache",
        u"Macjonc",
        u"Coing",
        u"Cormier",
        u"Rouleau",
    ],
    3: [
        u"Raiponce",
        u"Turneps",
        u"Chicorée",
        u"Nèfle",
        u"Cochon",
        u"Mâche",
        u"Chou-fleur",
        u"Miel",
        u"Genièvre",
        u"Pioche",
        u"Cire",
        u"Raifort",
        u"Cèdre",
        u"Sapin",
        u"Chevreuil",
        u"Ajonc",
        u"Cyprès",
        u"Lierre",
        u"Sabine",
        u"Hoyau",
        u"Érable à sucre",
        u"Bruyère",
        u"Roseau",
        u"Oseille",
        u"Grillon",
        u"Pignon",
        u"Liège",
        u"Truffe",
        u"Olive",
        u"Pelle",
    ],
    4: [
        u"Tourbe",
        u"Houille",
        u"Bitume",
        u"Soufre",
        u"Chien",
        u"Lave",
        u"Terre végétale",
        u"Fumier",
        u"Salpêtre",
        u"Fléau",
        u"Granit",
        u"Argile",
        u"Ardoise",
        u"Grès",
        u"Lapin",
        u"Silex",
        u"Marne",
        u"Pierre à chaux",
        u"Marbre",
        u"Van",
        u"Pierre à plâtre",
        u"Sel",
        u"Fer",
        u"Cuivre",
        u"Chat",
        u"Étain",
        u"Plomb",
        u"Zinc",
        u"Mercure",
        u"Crible",
    ],
    5: [
        u"Lauréole",
        u"Mousse",
        u"Fragon",
        u"Perce-neige",
        u"Taureau",
        u"Laurier-thym",
        u"Amadouvier",
        u"Mézéréon",
        u"Peuplier",
        u"Coignée",
        u"Ellébore",
        u"Brocoli",
        u"Laurier",
        u"Avelinier",
        u"Vache",
        u"Buis",
        u"Lichen",
        u"If",
        u"Pulmonaire",
        u"Serpette",
        u"Thlaspi",
        u"Thimelé",
        u"Chiendent",
        u"Trainasse",
        u"Lièvre",
        u"Guède",
        u"Noisetier",
        u"Cyclamen",
        u"Chélidoine",
        u"Traîneau",
    ],
    6: [
        u"Tussilage",
        u"Cornouiller",
        u"Violier",
        u"Troène",
        u"Bouc",
        u"Asaret",
        u"Alaterne",
        u"Violette",
        u"Marceau",
        u"Bêche",
        u"Narcisse",
        u"Orme",
        u"Fumeterre",
        u"Vélar",
        u"Chèvre",
        u"Épinard",
        u"Doronic",
        u"Mouron",
        u"Cerfeuil",
        u"Cordeau",
        u"Mandragore",
        u"Persil",
        u"Cochléaria",
        u"Pâquerette",
        u"Thon",
        u"Pissenlit",
        u"Sylvie",
        u"Capillaire",
        u"Frêne",
        u"Plantoir",
    ],
    7: [
        u"Primevère",
        u"Platane",
        u"Asperge",
        u"Tulipe",
        u"Poule",
        u"Bette",
        u"Bouleau",
        u"Jonquille",
        u"Aulne",
        u"Couvoir",
        u"Pervenche",
        u"Charme",
        u"Morille",
        u"Hêtre",
        u"Abeille",
        u"Laitue",
        u"Mélèze",
        u"Ciguë",
        u"Radis",
        u"Ruche",
        u"Gainier",
        u"Romaine",
        u"Marronnier",
        u"Roquette",
        u"Pigeon",
        u"Lilas",
        u"Anémone",
        u"Pensée",
        u"Myrtille",
        u"Greffoir",
    ],
    8: [
        u"Rose",
        u"Chêne",
        u"Fougère",
        u"Aubépine",
        u"Rossignol",
        u"Ancolie",
        u"Muguet",
        u"Champignon",
        u"Hyacinthe",
        u"Râteau",
        u"Rhubarbe",
        u"Sainfoin",
        u"Bâton d'or",
        u"Chamerisier",
        u"Ver à soie",
        u"Consoude",
        u"Pimprenelle",
        u"Corbeille d'or",
        u"Arroche",
        u"Sarcloir",
        u"Statice",
        u"Fritillaire",
        u"Bourrache",
        u"Valériane",
        u"Carpe",
        u"Fusain",
        u"Civette",
        u"Buglosse",
        u"Sénevé",
        u"Houlette",
    ],
    9: [
        u"Luzerne",
        u"Hémérocalle",
        u"Trèfle",
        u"Angélique",
        u"Canard",
        u"Mélisse",
        u"Fromental",
        u"Martagon",
        u"Serpolet",
        u"Faux",
        u"Fraise",
        u"Bétoine",
        u"Pois",
        u"Acacia",
        u"Caille",
        u"Œillet",
        u"Sureau",
        u"Pavot",
        u"Tilleul",
        u"Fourche",
        u"Barbeau",
        u"Camomille",
        u"Chèvrefeuille",
        u"Caille-lait",
        u"Tanche",
        u"Jasmin",
        u"Verveine",
        u"Thym",
        u"Pivoine",
        u"Chariot",
    ],
    10: [
        u"Seigle",
        u"Avoine",
        u"Oignon",
        u"Véronique",
        u"Mulet",
        u"Romarin",
        u"Concombre",
        u"Échalote",
        u"Absinthe",
        u"Faucille",
        u"Coriandre",
        u"Artichaut",
        u"Girofle",
        u"Lavande",
        u"Chamois",
        u"Tabac",
        u"Groseille",
        u"Gesse",
        u"Cerise",
        u"Parc",
        u"Menthe",
        u"Cumin",
        u"Haricot",
        u"Orcanète",
        u"Pintade",
        u"Sauge",
        u"Ail",
        u"Vesce",
        u"Blé",
        u"Chalémie",
    ],
    11: [
        u"Épeautre",
        u"Bouillon blanc",
        u"Melon",
        u"Ivraie",
        u"Bélier",
        u"Prêle",
        u"Armoise",
        u"Carthame",
        u"Mûre",
        u"Arrosoir",
        u"Panic",
        u"Salicorne",
        u"Abricot",
        u"Basilic",
        u"Brebis",
        u"Guimauve",
        u"Lin",
        u"Amande",
        u"Gentiane",
        u"Écluse",
        u"Carline",
        u"Câprier",
        u"Lentille",
        u"Aunée",
        u"Loutre",
        u"Myrte",
        u"Colza",
        u"Lupin",
        u"Coton",
        u"Moulin",
    ],
    12: [
        u"Prune",
        u"Millet",
        u"Lycoperdon",
        u"Escourgeon",
        u"Saumon",
        u"Tubéreuse",
        u"Sucrion",
        u"Apocyn",
        u"Réglisse",
        u"Échelle",
        u"Pastèque",
        u"Fenouil",
        u"Épine vinette",
        u"Noix",
        u"Truite",
        u"Citron",
        u"Cardère",
        u"Nerprun",
        u"Tagette",
        u"Hotte",
        u"Églantier",
        u"Noisette",
        u"Houblon",
        u"Sorgho",
        u"Écrevisse",
        u"Bigarade",
        u"Verge d'or",
        u"Maïs",
        u"Marron",
        u"Panier",
    ],
    13: [
        u"La Fête de la Vertu",
        u"La Fête du Génie",
        u"La Fête du Travail",
        u"La Fête de l'Opinion",
        u"La Fête des Récompenses",
        u"La Fête de la Révolution",
    ]
}
